"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BackpackStrip = void 0;
var __1 = require("..");
var constants_1 = require("../constants");
var pixel_1 = __importDefault(require("../pixel"));
var strip_1 = require("./strip");
var BackpackStrip = /** @class */ (function (_super) {
    __extends(BackpackStrip, _super);
    function BackpackStrip(opts) {
        var _a;
        var _this = _super.call(this, opts) || this;
        var MAX_PIXELS = 500; // based on # bytes available in firmata
        var strip_length = opts.length || 6; // just an arbitrary val
        var strip_definition = opts.strips || [];
        var color_order = opts.color_order || __1.COLOR_ORDER.GRB; // default GRB
        var io = opts.firmata || ((_a = opts.board) === null || _a === void 0 ? void 0 : _a.io);
        if (!opts.address) {
            opts.address = constants_1.I2C_DEFAULT;
        }
        if (io === undefined) {
            var err = new Error('An IO object is required to I2C controller');
            err.name = 'NoIOError';
            throw err;
        }
        var builtStripInstances = [];
        // work out the map of strips and pixels.
        if (typeof (strip_definition[0]) === 'undefined') {
            // there is nothing specified so it's probably a single strip
            // using the length and colour type.
            builtStripInstances.push({
                color_order: color_order,
                length: strip_length
            });
        }
        else if (!isNaN(+strip_definition[0])) {
            // we have the array of pin lengths but do we have the colour
            for (var i = 0; i < strip_definition.length; i++) {
                var len = strip_definition[i];
                builtStripInstances[i] = {
                    color_order: color_order,
                    length: +len
                };
            }
        }
        else {
            strip_definition.forEach(function (stripValue) { return builtStripInstances.push(stripValue); });
        }
        // put in check if it's gone over.
        if (builtStripInstances.length > constants_1.MAX_STRIPS) {
            var err = new RangeError('Maximum number of strips ' + constants_1.MAX_STRIPS + ' exceeded');
            _this.emit('error', err);
        }
        var total_length = 0;
        builtStripInstances.forEach(function (data) {
            total_length += data.length;
        });
        // put in check if there are too many pixels.
        if (total_length > MAX_PIXELS) {
            var err = new RangeError('Maximum number of pixels ' + MAX_PIXELS + ' exceeded');
            _this.emit('error', err);
        }
        _this.length = total_length;
        for (var i = 0; i < total_length; i++) {
            _this.pixels.push(pixel_1.default({
                addr: i,
                io: io,
                controller: 'I2CBACKPACK',
                i2c_address: opts.address,
                strip: _this
            }));
        }
        _this.io = io;
        _this.i2c_address = opts.address;
        // now send the config message with length and data point.
        var data = [];
        data.push(constants_1.PIXEL_CONFIG);
        builtStripInstances.forEach(function (strip) {
            var colorOrder = strip.color_order;
            var pinLocation = strip.pin || 0;
            var pinOrColorInfo = colorOrder ? (colorOrder << 5) : pinLocation;
            data.push(pinOrColorInfo);
            data.push(strip.length & constants_1.FIRMATA_7BIT_MASK);
            data.push((strip.length >> 7) & constants_1.FIRMATA_7BIT_MASK);
        });
        // send the I2C config message.
        _this.io.i2cConfig(opts);
        process.nextTick(function () {
            try {
                io.i2cWrite(opts.address, data);
            }
            catch (e) {
                if (e instanceof Error && e.name == 'EIO') {
                    _this.emit('np_i2c_write_error', data);
                }
            }
            process.nextTick(function () {
                _this.emit('ready', null);
            });
        });
        return _this;
    }
    BackpackStrip.prototype.show = function () {
        try {
            this.io.i2cWrite(this.i2c_address, [constants_1.PIXEL_SHOW]);
        }
        catch (e) {
            if (e instanceof Error && e.name == 'EIO') {
                this.emit('np_i2c_write_error', 'PIXEL_SHOW');
            }
        }
    };
    BackpackStrip.prototype.stripColor = function (color) {
        var data = [];
        data[0] = constants_1.PIXEL_SET_STRIP;
        data[1] = color & constants_1.FIRMATA_7BIT_MASK;
        data[2] = (color >> 7) & constants_1.FIRMATA_7BIT_MASK;
        data[3] = (color >> 14) & constants_1.FIRMATA_7BIT_MASK;
        data[4] = (color >> 21) & constants_1.FIRMATA_7BIT_MASK;
        try {
            this.io.i2cWrite(this.i2c_address, data);
        }
        catch (e) {
            if (e instanceof Error && e.name == 'EIO') {
                this.emit('np_i2c_write_error', data);
            }
        }
    };
    BackpackStrip.prototype._shift = function (amt, direction, wrap) {
        // shifts the strip in the appropriate direction.
        var wrap_val = wrap ? constants_1.PIXEL_SHIFT_WRAP : 0;
        var data = [];
        data[0] = constants_1.PIXEL_SHIFT;
        data[1] = (amt | direction | wrap_val) & constants_1.FIRMATA_7BIT_MASK;
        try {
            this.io.i2cWrite(this.i2c_address, data);
        }
        catch (e) {
            if (e instanceof Error && e.name == 'EIO') {
                this.emit('np_i2c_write_error', data);
            }
        }
    };
    return BackpackStrip;
}(strip_1.Strip));
exports.BackpackStrip = BackpackStrip;
