"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FirmataStrip = void 0;
var __1 = require("..");
var constants_1 = require("../constants");
var pixel_1 = __importDefault(require("../pixel"));
var strip_1 = require("./strip");
var FirmataStrip = /** @class */ (function (_super) {
    __extends(FirmataStrip, _super);
    function FirmataStrip(opts) {
        var _a;
        var _this = _super.call(this, opts) || this;
        var MAX_PIXELS = 216; // based on # bytes available in firmata
        var strip_length = opts.length || 6; // just an arbitrary val
        var data_pin = opts.data || constants_1.PIN_DEFAULT;
        var color_order = opts.color_order || __1.COLOR_ORDER.GRB; // default GRB
        var strip_definition = opts.strips || [];
        var skip_firmware_check = !!opts.skip_firmware_check;
        // do firmata / IO checks
        var firmata = opts.firmata || ((_a = opts === null || opts === void 0 ? void 0 : opts.board) === null || _a === void 0 ? void 0 : _a.io);
        // check if we're *still* undefined
        if (firmata === undefined) {
            var err = new Error('A firmata or board object is required');
            err.name = 'NoFirmataError';
            throw err;
        }
        if (firmata.firmware.name !== 'node_pixel_firmata.ino' && !skip_firmware_check) {
            var err = new Error('Please upload NodePixel Firmata to the board');
            err.name = 'IncorrectFirmataVersionError';
            throw err;
        }
        // figure out where we are writing to
        var port = firmata.transport || firmata.sp || firmata;
        if (port.write === undefined) {
            var err = new Error('Node Pixel FIRMATA controller requires IO that can write out');
            err.name = 'NoWritablePortError';
            throw err;
        }
        _this.port = port;
        var builtStripInstances = [];
        // work out the map of strips and pixels.
        if (typeof (strip_definition[0]) === 'undefined') {
            // there is nothing specified so it's probably a single strip
            // using the length and colour type.
            builtStripInstances.push({
                color_order: color_order,
                pin: data_pin,
                length: strip_length
            });
        }
        else if (!isNaN(+strip_definition[0])) {
            // we have the array of pin lengths but do we have the colour
            for (var i = 0; i < strip_definition.length; i++) {
                var len = strip_definition[i];
                builtStripInstances[i] = {
                    pin: data_pin,
                    color_order: color_order,
                    length: +len
                };
            }
        }
        else {
            strip_definition.forEach(function (stripValue) { return builtStripInstances.push(stripValue); });
        }
        // put in check if it's gone over value
        if (builtStripInstances.length > constants_1.MAX_STRIPS) {
            var err = new RangeError('Maximum number of strips ' + constants_1.MAX_STRIPS + ' exceeded');
            _this.emit('error', err);
        }
        var total_length = 0;
        builtStripInstances.forEach(function (data) {
            total_length += data.length;
        });
        // put in check if there are too many pixels.
        if (total_length > MAX_PIXELS) {
            var err = new RangeError('Maximum number of pixels ' + MAX_PIXELS + ' exceeded');
            _this.emit('error', err);
        }
        _this.length = total_length;
        for (var i = 0; i < total_length; i++) {
            _this.pixels.push(pixel_1.default({
                addr: i,
                firmata: firmata,
                port: port,
                controller: 'FIRMATA',
                strip: _this
            }));
        }
        // now send the config message with length and data point.
        var data = [];
        data[0] = constants_1.START_SYSEX;
        data[1] = constants_1.PIXEL_COMMAND;
        data[2] = constants_1.PIXEL_CONFIG;
        builtStripInstances.forEach(function (strip) {
            var colorOrder = strip.color_order;
            var pinLocation = strip.pin || 0;
            var pinOrColorInfo = colorOrder ? (colorOrder << 5) : pinLocation;
            data.push(pinOrColorInfo);
            data.push(strip.length & constants_1.FIRMATA_7BIT_MASK);
            data.push((strip.length >> 7) & constants_1.FIRMATA_7BIT_MASK);
        });
        data.push(constants_1.END_SYSEX);
        _this.port.write(Buffer.from(data), function (error) {
            var err;
            if (error) {
                err = error;
                _this.emit('error', err);
            }
            // there is a weird bug in OSX which sometimes causes
            // a segfault if you try to write to fast. As such
            // just delay the ready event by 1msec because even this
            // is faster than hooman will perceive as a delay
            setTimeout(function () {
                _this.emit('ready', err);
            }, 1);
        });
        return _this;
    }
    FirmataStrip.prototype.show = function () {
        var data = [];
        data[0] = constants_1.START_SYSEX;
        data[1] = constants_1.PIXEL_COMMAND;
        data[2] = constants_1.PIXEL_SHOW;
        data[3] = constants_1.END_SYSEX;
        // now just write that to the port and it should show the frame.
        this.port.write(Buffer.from(data));
    };
    FirmataStrip.prototype.stripColor = function (color) {
        // colour work is already done this just sets it the appropriate
        // way.
        var data = [];
        data[0] = constants_1.START_SYSEX;
        data[1] = constants_1.PIXEL_COMMAND;
        data[2] = constants_1.PIXEL_SET_STRIP;
        data[3] = color & constants_1.FIRMATA_7BIT_MASK;
        data[4] = (color >> 7) & constants_1.FIRMATA_7BIT_MASK;
        data[5] = (color >> 14) & constants_1.FIRMATA_7BIT_MASK;
        data[6] = (color >> 21) & constants_1.FIRMATA_7BIT_MASK;
        data[7] = constants_1.END_SYSEX;
        this.port.write(Buffer.from(data));
    };
    FirmataStrip.prototype._shift = function (amt, direction, wrap) {
        // shifts the strip in the appropriate direction.
        var wrap_val = wrap ? constants_1.PIXEL_SHIFT_WRAP : 0;
        var data = [];
        data[0] = constants_1.START_SYSEX;
        data[1] = constants_1.PIXEL_COMMAND;
        data[2] = constants_1.PIXEL_SHIFT;
        data[3] = (amt | direction | wrap_val) & constants_1.FIRMATA_7BIT_MASK;
        data[4] = constants_1.END_SYSEX;
        this.port.write(Buffer.from(data));
    };
    return FirmataStrip;
}(strip_1.Strip));
exports.FirmataStrip = FirmataStrip;
