"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Strip = void 0;
var events_1 = require("events");
var color_string_1 = __importDefault(require("color-string"));
var constants_1 = require("../constants");
var utils_1 = require("../utils");
var Strip = /** @class */ (function (_super) {
    __extends(Strip, _super);
    function Strip(opts) {
        var _this = _super.call(this) || this;
        _this.gamma = opts.gamma || constants_1.GAMMA_DEFAULT;
        _this.gtable = utils_1.buildGammaTable(256, _this.gamma);
        _this.whiteCap = _this.createWhiteCap(opts.whiteCap);
        _this.pixels = [];
        _this.length = 0;
        return _this;
    }
    Strip.prototype.createWhiteCap = function (userWhiteCaps) {
        var _this = this;
        if ((userWhiteCaps === null || userWhiteCaps === void 0 ? void 0 : userWhiteCaps.length) === 3) {
            return userWhiteCaps.map(function (value) { return value.gamma ? value : __assign(__assign({}, value), { gamma: _this.gamma, g_table: utils_1.buildGammaTable(256, _this.gamma) }); });
        }
    };
    Strip.prototype.pixel = function (addr) {
        return this.pixels && this.pixels[addr];
    };
    Strip.prototype.getLength = function () {
        return this.pixels ? this.pixels.length : 0;
    };
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    Strip.prototype.stripColor = function (color) {
        return;
    };
    Strip.prototype.show = function () {
        return;
    };
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    Strip.prototype._shift = function (amt, direction, wrap) {
        return;
    };
    Strip.prototype.shift = function (amt, direction, wrap) {
        // public version of the shift function independent of the controller.
        // this looks after the actual internal shifting of the pixels within the
        // js side and then calls the controller to mirror the same function.
        if (!this.pixels)
            return;
        if (amt > 0) {
            // take a copy of the pixels at the end that is being towards
            var start_element = 0;
            if (direction == constants_1.SHIFT_FORWARD) {
                start_element = this.length - amt;
            }
            var tmp_pixels = this.pixels.splice(start_element, amt);
            while (tmp_pixels.length > 0) {
                var px = tmp_pixels.pop();
                if (!px) {
                    break;
                }
                // set the pixel off if not wrapping.
                if (!wrap) {
                    px.color('#000');
                }
                if (direction == constants_1.SHIFT_FORWARD) {
                    this.pixels.unshift(px);
                }
                else {
                    this.pixels.push(px);
                }
            }
            // renumber the items so the addresses are correct for display
            this.pixels.forEach(function (px, index) {
                px.internalPixel && px.internalPixel.address ? px.internalPixel.address = index : undefined;
            });
            // now get the firmware to update appropriately as well.
            this._shift(amt, direction, wrap);
        }
    };
    Strip.prototype.color = function (color, brightness) {
        var _a;
        // sets the color of the entire strip
        // use a particular form to set the color either
        // color = hex value or named colors
        // or set color null and set opt which is an object as {rgb: [rx, gx, bx]}
        // values where x is an 8-bit value (0-255);
        if (!this.pixels) {
            return;
        }
        var stripcolor;
        if (color) {
            // use text to determine the color
            if (Array.isArray(color)) {
                // we have an RGB array value
                stripcolor = color;
            }
            else {
                stripcolor = (_a = color_string_1.default.get(color)) === null || _a === void 0 ? void 0 : _a.value;
            }
        }
        if (stripcolor) {
            // fill out the values for the pixels and then update the strip
            for (var i = 0; i < this.pixels.length; i++) {
                this.pixels[i].color(color, { sendmsg: false });
            }
            // set the whole strip color to the appropriate int value
            var builtColor = utils_1.colorValue(stripcolor, this.gtable);
            if (this.whiteCap) {
                if (brightness !== undefined) {
                    builtColor = utils_1.normalizeColorWithBrightness(stripcolor, brightness, this.whiteCap);
                }
                else {
                    builtColor = utils_1.normalizeColor(stripcolor, this.whiteCap);
                }
            }
            this.stripColor(builtColor);
        }
        else {
            console.log("Supplied colour couldn't be parsed: " + color);
        }
    };
    Strip.prototype.colour = function (color, brightness) {
        return this.color(color, brightness);
    };
    Strip.prototype.off = function () {
        this.color([0, 0, 0]);
        this.show();
        return this;
    };
    return Strip;
}(events_1.EventEmitter));
exports.Strip = Strip;
