"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeColorWithBrightness = exports.normalizeColor = exports.colorValue = exports.buildGammaTable = exports.rgbToHex = void 0;
var rgbToHex = function (colorArray) { return ((colorArray[0] << 16) + (colorArray[1] << 8) + (colorArray[2])); };
exports.rgbToHex = rgbToHex;
// helper function for building gamma values
function buildGammaTable(steps, gamma) {
    // used to build a gamma table for a particular value
    return Array(steps).fill(0).map(function (_, index) { return Math.floor(Math.pow((index / 255.0), gamma) * 255 + 0.5); });
}
exports.buildGammaTable = buildGammaTable;
function colorValue(colors, g_table) {
    return exports.rgbToHex([g_table[colors[0]], g_table[colors[1]], g_table[colors[2]]]);
}
exports.colorValue = colorValue;
function normalizeColor(colors, whiteCap) {
    var gammaCorrectedColor = [colors[0], colors[1], colors[2]].map(function (current, index) { return whiteCap[index].g_table[current]; }).map(function (current, index) { return Math.round((current / 255) * whiteCap[index].maximum); });
    return exports.rgbToHex([gammaCorrectedColor[0], gammaCorrectedColor[1], gammaCorrectedColor[2]]);
}
exports.normalizeColor = normalizeColor;
function normalizeColorWithBrightness(colors, brightness, whiteCap) {
    var gammaCorrectedColor = [colors[0], colors[1], colors[2]].map(function (current, index) { return whiteCap[index].g_table[current]; }).map(function (current, index) { return Math.round((current / 255) * whiteCap[index].maximum); }).map(function (current) { return Math.round(Math.max(Math.min(current * brightness, 255), 0)); });
    return exports.rgbToHex([gammaCorrectedColor[0], gammaCorrectedColor[1], gammaCorrectedColor[2]]);
}
exports.normalizeColorWithBrightness = normalizeColorWithBrightness;
